"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var generateModel_1 = require("./generateModel");
exports.evaluateType = function (field) {
    var propertyName = field.name;
    var isOptional = true;
    var isArray = false;
    var isArrayOptional = false;
    var isEdge = false;
    var isScalar = false;
    var typeName = '';
    function getFieldInfos(type) {
        if (propertyName === 'edges')
            isEdge = true;
        if (type.kind === 'NON_NULL' || type.kind === 'LIST') {
            if (type.kind === 'LIST') {
                isArray = true;
                if (isOptional)
                    isArrayOptional = true;
            }
            if (type.kind === 'NON_NULL' && !isArrayOptional)
                isOptional = false;
            return getFieldInfos(type.ofType);
        }
        else {
            if (type.kind === 'SCALAR') {
                isScalar = true;
            }
            typeName = type.name;
        }
    }
    getFieldInfos(field.type);
    return {
        isOptional: isOptional,
        isArray: isArray,
        isArrayOptional: isArrayOptional,
        isEdge: isEdge,
        isScalar: isScalar,
        typeName: typeName,
    };
};
exports.getOneTSType = function (_a) {
    var field = _a.field, prefix = _a.prefix, suffix = _a.suffix;
    var _b = exports.evaluateType(field), isScalar = _b.isScalar, typeName = _b.typeName, isArray = _b.isArray;
    return isScalar
        ? generateModel_1.scalarList[typeName]
        : "" + (prefix ? prefix : '') + typeName + (suffix ? suffix : '') + (isArray ? '[]' : '');
};
exports.getObjectTSInterfaces = function (object, prefix, suffix) {
    var ObjectName = object.name;
    var fieldsKey = object.kind === 'OBJECT' || object.kind === 'INTERFACE' ? 'fields' : 'inputFields';
    var generatedFields = object[fieldsKey].map(function (field) {
        var propertyName = field.name;
        var _a = exports.evaluateType(field), isArray = _a.isArray, isOptional = _a.isOptional, isScalar = _a.isScalar, typeName = _a.typeName;
        var TStypeName = isScalar
            ? generateModel_1.scalarList[typeName]
            : (prefix ? prefix : '') + typeName + (suffix ? suffix : '');
        var generatedProperty = "" + propertyName + (isOptional ? '?' : '') + ": " + TStypeName + (isArray ? '[]' : '') + ";";
        return generatedProperty;
    });
    var generatedInterface = "interface " + (prefix ? prefix : '') + ObjectName + (suffix ? suffix : '') + " {\n        " + generatedFields.join('\n') + "\n      }\n    ";
    return generatedInterface;
};
exports.getQueriesArgsTSInterfaces = function (object, prefix, suffix) {
    var ObjectName = object.name;
    var parsedSuffix = 'Args' + (suffix ? suffix : '');
    var generatedFields = object.args.map(function (field) {
        var propertyName = field.name;
        var _a = exports.evaluateType(field), isArray = _a.isArray, isOptional = _a.isOptional, isScalar = _a.isScalar, typeName = _a.typeName;
        var TStypeName = isScalar
            ? generateModel_1.scalarList[typeName]
            : (prefix ? prefix : '') + typeName + (suffix ? suffix : '');
        var generatedProperty = "" + propertyName + (isOptional ? '?' : '') + ": " + TStypeName + (isArray ? '[]' : '') + ";";
        return generatedProperty;
    });
    var generatedInterface = "interface " + (prefix ? prefix : '') + ObjectName + (parsedSuffix ? parsedSuffix : '') + " {\n        " + generatedFields.join('\n') + "\n      }\n    ";
    return generatedInterface;
};
exports.getObjectGQLTypesArgs = function (field) {
    var _a = exports.evaluateType(field), isArray = _a.isArray, isArrayOptional = _a.isArrayOptional, isEdge = _a.isEdge, isOptional = _a.isOptional, isScalar = _a.isScalar, typeName = _a.typeName;
    var generatedType = "" + (isArray ? '[' : '') + typeName + (isOptional ? '' : '!') + (isArray ? ']' : '') + (isArrayOptional ? '!' : '');
    return generatedType;
};
exports.buildMethod = function (data, type, prefix, suffix) {
    var hasArgs = data.args.length > 0;
    var methodName = data.name;
    var _a = data.args.reduce(function (acc, arg) {
        var argName = arg.name;
        var _a = exports.evaluateType(arg), isScalar = _a.isScalar, isOptional = _a.isOptional, isArray = _a.isArray;
        var type = exports.getObjectGQLTypesArgs(arg);
        var tsArg = exports.getOneTSType({ field: arg, prefix: prefix, suffix: suffix });
        acc.$args.push("$" + argName + ": " + type);
        acc.args.push(argName + ": $" + argName);
        acc.variables.push(argName);
        acc.tsArgs.push("" + argName + (isOptional ? '?' : '') + ": " + tsArg + ";");
        return acc;
    }, {
        $args: [],
        args: [],
        variables: [],
        tsArgs: [],
    }), $args = _a.$args, args = _a.args, variables = _a.variables, tsArgs = _a.tsArgs;
    var _b = exports.evaluateType(data), isScalar = _b.isScalar, isEdge = _b.isEdge;
    var returnedType = exports.getOneTSType({ field: data, prefix: prefix, suffix: suffix });
    var renderedArgs = '';
    if (tsArgs.length) {
        renderedArgs = "{" + tsArgs.join('\n') + "}";
    }
    var scalarFunction = '';
    var nonScalarFunction = '';
    if (type.little === 'mutation') {
        scalarFunction = "\n    const mutation = graphQlTag`\n      mutation " + methodName + " " + (hasArgs ? "(" + $args.join(',') + ")" : '') + " {\n        " + methodName + (hasArgs ? "(" + args.join(',') + ")" : '') + "\n      }`,\n    return abortableMutation<" + returnedType + ">(mutation);\n  ";
        nonScalarFunction = "\n  return {\n    $fragment: (fragment: string | DocumentNode) => {\n      const { isString, isFragment, fragmentName } = guessFragmentType(fragment);\n      const mutation = graphQlTag`\n         mutation " + methodName + " " + (hasArgs ? "(" + $args.join(',') + ")" : '') + " {\n        " + methodName + (hasArgs ? "(" + args.join(',') + ")" : '') + " {\n          " + (isEdge ? '' : "${isString ? fragment : '...' + fragmentName}") + "\n          \n        }\n      } ${isFragment? fragment: ''}\n      `\n\n      return abortableMutation<" + returnedType + (renderedArgs.length ? ',' + renderedArgs : '') + ">(mutation);\n    }\n  }\n";
    }
    else {
        scalarFunction = "\n    const query = graphQlTag`\n      query " + methodName + " " + (hasArgs ? "(" + $args.join(',') + ")" : '') + " {\n        " + methodName + (hasArgs ? "(" + args.join(',') + ")" : '') + "\n      }`,\n      return abortableQuery<" + returnedType + (renderedArgs.length ? ',' + renderedArgs : '') + ">(query);\n    ";
        nonScalarFunction = "\n    return {\n      $fragment: (fragment: string | DocumentNode) => {\n        const { isString, isFragment, fragmentName } = guessFragmentType(fragment);\n        const query = graphQlTag`\n           query " + methodName + " " + (hasArgs ? "(" + $args.join(',') + ")" : '') + " {\n          " + methodName + (hasArgs ? "(" + args.join(',') + ")" : '') + " {\n            " + (isEdge ? '' : "${isString ? fragment : '...' + fragmentName}") + "\n            \n          }\n        } ${isFragment? fragment: ''}\n        `\n        return abortable" + type.high + "<" + returnedType + (renderedArgs.length ? ',' + renderedArgs : '') + ">(query);\n      }\n    }\n  ";
    }
    var withArgs = tsArgs.length ? 'WithArgs' : '';
    var template = "\n    " + methodName + ": (): " + (isScalar
        ? "Abordable" + type.high + withArgs + "<" + returnedType + (renderedArgs.length ? ',' + renderedArgs : '') + ">"
        : "Fragmentable" + type.high + withArgs + "<" + returnedType + (renderedArgs.length ? ',' + renderedArgs : '') + ">") + " => {\n        " + (isScalar ? scalarFunction : nonScalarFunction) + "\n      }\n    ,";
    return template;
};
